@@ -3,7 +3,7 @@
 # MODEVAL.TCL, module evaluation procedures
 # Copyright (C) 2002-2004 Mark Lakata
 # Copyright (C) 2004-2017 Kent Mein
-# Copyright (C) 2016-2021 Xavier Delaruelle
+# Copyright (C) 2016-2022 Xavier Delaruelle
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -222,7 +222,6 @@ proc sortModulePerLoadedAndDepOrder {modlist {nporeq 0} {loading 0}} {
 # return list of loaded modules having an unmet requirement on passed mod
 # and their recursive dependent
 proc getUnmetDependentLoadedModuleList {modnamevr} {
-   reportDebug "get dependent of upcoming loaded '$modnamevr'"
    set unmetdeplist {}
    set depmodlist {}
    defineModEqProc [isIcase] [getConf extended_default] 1
@@ -451,8 +450,10 @@ proc isModuleUnloadable {mod {unmodlist {}}} {
       set unmodlist [getUnloadingModuleList]
    }
 
-   # loaded tag means module was not auto-loaded
-   if {[isModuleTagged $mod loaded 1]} {
+   # loaded tag means module was not auto-loaded. keep-loaded means module is
+   # not auto-unloadable even if auto-loaded
+   if {[isModuleTagged $mod loaded 1] || [isModuleTagged $mod keep-loaded\
+      1]} {
       set ret 0
    } else {
       # mod is unloadable if all its dependent are unloaded or unloading
@@ -470,8 +471,6 @@ proc isModuleUnloadable {mod {unmodlist {}}} {
 # gets the list of all loaded modules which are required by passed modlist
 # ordered by load position.
 proc getRequiredLoadedModuleList {modlist} {
-   reportDebug "get mods required by '$modlist'"
-
    # search over all list of loaded modules, starting with passed module
    # list, then adding in turns their requirements
    set fulllist $modlist
@@ -491,8 +490,6 @@ proc getRequiredLoadedModuleList {modlist} {
 # they have been loaded automatically and are not depended (mandatory or
 # optionally) by other module
 proc getUnloadableLoadedModuleList {modlist} {
-   reportDebug "get unloadable mods once '$modlist' unloaded"
-
    # search over all list of unloaded modules, starting with passed module
    # list, then adding in turns unloadable requirements
    set fulllist $modlist
@@ -534,17 +531,18 @@ proc getSavedSettingsStackDepth {} {
 # correct set in case of failure
 proc pushSettings {} {
    foreach var {env g_clearedEnvVars g_Aliases g_stateEnvVars g_stateAliases\
-      g_stateFunctions g_Functions g_newXResources g_delXResources\
-      g_loadedModules g_loadedModuleFiles g_loadedModuleVariant\
-      g_loadedModuleConflict g_loadedModulePrereq g_loadedModuleAltname\
-      g_loadedModuleAutoAltname g_loadedModuleAliasAltname g_moduleDepend\
-      g_dependHash g_moduleNPODepend g_dependNPOHash g_prereqViolation\
+      g_stateFunctions g_Functions g_stateCompletes g_Completes\
+      g_newXResources g_delXResources g_loadedModules g_loadedModuleFiles\
+      g_loadedModuleVariant g_loadedModuleConflict g_loadedModulePrereq\
+      g_loadedModuleAltname g_loadedModuleAutoAltname\
+      g_loadedModuleAliasAltname g_moduleDepend g_dependHash\
+      g_moduleNPODepend g_dependNPOHash g_prereqViolation\
       g_prereqNPOViolation g_conflictViolation g_moduleUnmetDep\
       g_unmetDepHash g_moduleEval g_moduleHiddenEval} {
       lappend ::g_SAVE_$var [array get ::$var]
    }
    # save non-array variable and indication if it was set
-   foreach var {g_changeDir g_stdoutPuts g_return_text} {
+   foreach var {g_changeDir g_stdoutPuts g_prestdoutPuts g_return_text} {
       if {[info exists ::$var]} {
          lappend ::g_SAVE_$var [list 1 [set ::$var]]
       } else {
@@ -557,12 +555,13 @@ proc pushSettings {} {
 proc popSettings {} {
    set flushedid [getSavedSettingsStackDepth]
    foreach var {env g_clearedEnvVars g_Aliases g_stateEnvVars g_stateAliases\
-      g_stateFunctions g_Functions g_newXResources g_delXResources\
-      g_changeDir g_stdoutPuts g_return_text\
-      g_loadedModules g_loadedModuleFiles g_loadedModuleVariant\
-      g_loadedModuleConflict g_loadedModulePrereq g_loadedModuleAltname\
-      g_loadedModuleAutoAltname g_loadedModuleAliasAltname g_moduleDepend\
-      g_dependHash g_moduleNPODepend g_dependNPOHash g_prereqViolation\
+      g_stateFunctions g_Functions g_stateCompletes g_Completes\
+      g_newXResources g_delXResources g_changeDir g_stdoutPuts\
+      g_prestdoutPuts g_return_text g_loadedModules g_loadedModuleFiles\
+      g_loadedModuleVariant g_loadedModuleConflict g_loadedModulePrereq\
+      g_loadedModuleAltname g_loadedModuleAutoAltname\
+      g_loadedModuleAliasAltname g_moduleDepend g_dependHash\
+      g_moduleNPODepend g_dependNPOHash g_prereqViolation\
       g_prereqNPOViolation g_conflictViolation g_moduleUnmetDep\
       g_unmetDepHash g_moduleEval g_moduleHiddenEval} {
       set ::g_SAVE_$var [lrange [set ::g_SAVE_$var] 0 end-1]
@@ -572,11 +571,12 @@ proc popSettings {} {
 
 proc restoreSettings {} {
    foreach var {g_clearedEnvVars g_Aliases g_stateEnvVars g_stateAliases\
-      g_stateFunctions g_Functions g_newXResources g_delXResources\
-      g_loadedModules g_loadedModuleFiles g_loadedModuleVariant\
-      g_loadedModuleConflict g_loadedModulePrereq g_loadedModuleAltname\
-      g_loadedModuleAutoAltname g_loadedModuleAliasAltname g_moduleDepend\
-      g_dependHash g_moduleNPODepend g_dependNPOHash g_prereqViolation\
+      g_stateFunctions g_Functions g_stateCompletes g_Completes\
+      g_newXResources g_delXResources g_loadedModules g_loadedModuleFiles\
+      g_loadedModuleVariant g_loadedModuleConflict g_loadedModulePrereq\
+      g_loadedModuleAltname g_loadedModuleAutoAltname\
+      g_loadedModuleAliasAltname g_moduleDepend g_dependHash\
+      g_moduleNPODepend g_dependNPOHash g_prereqViolation\
       g_prereqNPOViolation g_conflictViolation g_moduleUnmetDep\
       g_unmetDepHash g_moduleEval g_moduleHiddenEval} {
       # clear current $var arrays
@@ -599,7 +599,7 @@ proc restoreSettings {} {
       }
    }
    # restore non-array variable if it was set
-   foreach var {g_changeDir g_stdoutPuts g_return_text} {
+   foreach var {g_changeDir g_stdoutPuts g_prestdoutPuts g_return_text} {
       if {[info exists ::$var]} {
          unset ::$var
       }
@@ -612,6 +612,59 @@ proc restoreSettings {} {
       (#[getSavedSettingsStackDepth])"
 }
 
+# load modules passed as args designated as requirement
+proc loadRequirementModuleList {tag_list args} {
+   # calling procedure must have already parsed module specification in args
+   foreach mod $args {
+      # get first loaded or loading mod in args list
+      if {[set loadedmod [getLoadedMatchingName $mod returnfirst]] ne {} ||\
+         [set loadedmod [getLoadedMatchingName $mod returnfirst 1]] ne {}} {
+         break
+      }
+   }
+
+   if {$loadedmod eq {}} {
+      set imax [llength $args]
+      set prereqloaded 0
+      # if prereq list specified, try to load first then
+      # try next if load of first module not successful
+      for {set i 0} {$i<$imax && $prereqloaded==0} {incr i 1} {
+         set arg [lindex $args $i]
+
+         # hold output of each evaluation until they are all done to drop
+         # those that failed if one succeed
+         set curholdid load-$i-$arg
+         lappendState reportholdid $curholdid
+         if {[catch {cmdModuleLoad reqlo 0 $tag_list $arg} errorMsg]} {
+            # if an error is raised, release output and rethrow the error
+            # (could be raised if no modulepath defined for instance)
+            lpopState reportholdid
+            lappend holdidlist $curholdid report
+            releaseHeldReport {*}$holdidlist
+            knerror $errorMsg
+         }
+         lpopState reportholdid
+
+         if {[is-loaded $arg]} {
+            set prereqloaded 1
+            # set previous reports to be dropped as this one succeed
+            if {[info exists holdidlist]} {
+               foreach {holdid action} $holdidlist {
+                  lappend newholdidlist $holdid drop
+               }
+               set holdidlist $newholdidlist
+            }
+         }
+         lappend holdidlist $curholdid report
+      }
+      # output held messages
+      releaseHeldReport {*}$holdidlist
+   } else {
+      # apply missing tag to first loaded module found
+      cmdModuleTag 0 0 $tag_list $loadedmod
+   }
+}
+
 # unload phase of a list of modules reload process
 proc reloadModuleListUnloadPhase {lmname {force 0} {errmsgtpl {}} {context\
    unload}} {
@@ -625,6 +678,8 @@ proc reloadModuleListUnloadPhase {lmname {force 0} {errmsgtpl {}} {context\
       set isuasked($mod) [isModuleTagged $mod loaded 1]
       # save variants set for modules
       set vr($mod) [getVariantList $mod 1 2]
+      # save extra tags set
+      set extratag($mod) [getExtraTagList $mod]
       # force unload even if requiring mods are not part of the unload list
       # (violation state) as modules are loaded again just after
       if {[cmdModuleUnload $context match 0 1 0 0 $mod]} {
@@ -646,15 +701,16 @@ proc reloadModuleListUnloadPhase {lmname {force 0} {errmsgtpl {}} {context\
       lpopState reloading_sticky
       lpopState reloading_supersticky
    }
-   return [list [array get isuasked] [array get vr]]
+   return [list [array get isuasked] [array get vr] [array get extratag]]
 }
 
 # load phase of a list of modules reload process
-proc reloadModuleListLoadPhase {lmname isuaskedlist vrlist {force 0}\
-   {errmsgtpl {}} {context load}} {
+proc reloadModuleListLoadPhase {lmname isuaskedlist vrlist extrataglist\
+   {force 0} {errmsgtpl {}} {context load}} {
    upvar $lmname lmlist
    array set isuasked $isuaskedlist
    array set vr $vrlist
+   array set extratag $extrataglist
 
    # loads are made with auto handling mode disabled to avoid disturbances
    # from a missing prereq automatically reloaded, so these module loads may
@@ -663,8 +719,9 @@ proc reloadModuleListLoadPhase {lmname isuaskedlist vrlist {force 0}\
    foreach mod $lmlist {
       # if an auto set default was excluded, module spec need parsing
       lassign [parseModuleSpecification 0 $mod {*}$vr($mod)] modnamevr
-      # reload module with user asked property preserved
-      if {[cmdModuleLoad $context $isuasked($mod) $modnamevr]} {
+      # reload module with user asked property and extra tags preserved
+      if {[cmdModuleLoad $context $isuasked($mod) $extratag($mod)\
+         $modnamevr]} {
          set errMsg [string map [list _MOD_ [getModuleDesignation spec\
             $modnamevr]] $errmsgtpl]
          if {$force} {
@@ -682,7 +739,7 @@ proc reloadModuleListLoadPhase {lmname isuaskedlist vrlist {force 0}\
 
 # test if loaded module 'mod' is sticky and if stickiness definition applies
 # to one of the reloading module
-proc isStickynessReloading {mod reloading_modlist {tag sticky}} {
+proc isStickinessReloading {mod reloading_modlist {tag sticky}} {
    set res 0
    set modname [getModuleNameAndVersFromVersSpec $mod]
    if {[isModuleTagged $modname $tag 1]} {
