@@ -9,6 +9,7 @@ if [ "${TESTSUITE_SHTOMOD_NOVAR:-x}" = 'x' ]; then
    export FOOEM=""
    export FOOSP="value "
    export FOOCB=va{ue
+   export FOOWC=va\*ue
    i=1
    while [ $# -gt 0 ]; do
       export FOOARG$i="$1"
@@ -23,6 +24,7 @@ if [ "${TESTSUITE_SHTOMOD_NOPATH:-x}" = 'x' ]; then
    export FOOPATHSP="/path/to/dir1 /path/to/dir2 /path/to/dir3"
    export FOOPATHSPEM="/path/to/dir1 /path/to/dir2 /path/to/dir3 "
    export FOOPATHCB=/path/to/dir1:/path/to/d{r2:/path/to/dir3
+   export FOOPATHWC=/path/to/dir1:/path/to/d\*r2:/path/to/dir3
 fi
 
 if [ "${TESTSUITE_SHTOMOD_PATHDUP:+x}" = 'x' ]; then
@@ -65,6 +67,15 @@ if [ "${TESTSUITE_SHTOMOD_NOFUNC:-x}" = 'x' ]; then
       echo b\\\"r
       echo f\'o
    }
+   funcwc() {
+      echo sou*sh
+   }
+fi
+
+if [ "${TESTSUITE_SHTOMOD_NOCOMP:-x}" = 'x' ]; then
+   complete -o default -F _cmd cmd
+   complete -j -P '"%' -S '"' mycmd
+   complete -u othercmd
 fi
 
 if [ "${TESTSUITE_SHTOMOD_NOCD:-x}" = 'x' ]; then
@@ -95,4 +106,20 @@ if [ "${TESTSUITE_SHTOMOD_EMPTYPREPEND:+x}" = 'x' ]; then
    TESTSUITE_SHTOMOD_EMPTYPREPEND=:$TESTSUITE_SHTOMOD_EMPTYPREPEND
 fi
 
+if [ "${TESTSUITE_SHTOMOD_MODULE:+x}" = 'x' ]; then
+   module load 2>/dev/null
+   if [ $? -ne 0 ]; then
+      eval $($TCLSH $MODULES_CMD sh autoinit)
+      autoinit=1
+   else
+      autoinit=0
+   fi
+   module config collection_target bar
+   module load setenv/1.0
+   if [ $autoinit -eq 1 ]; then
+      unset -f ml module _module_raw
+      unset MODULESHOME
+   fi
+fi
+
 # vim:set tabstop=3 shiftwidth=3 expandtab autoindent:
